@@ -15,9 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * along with this program; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -162,8 +161,6 @@ Database::Database(string_view path, int flags)
 
     int type = flags & DB_BACKEND_MASK_;
     switch (type) {
-	case DB_BACKEND_CHERT:
-	    throw FeatureUnavailableError("Chert backend no longer supported");
 	case DB_BACKEND_GLASS:
 #ifdef XAPIAN_HAS_GLASS_BACKEND
 	    internal = new GlassDatabase(path);
@@ -429,8 +426,6 @@ WritableDatabase::WritableDatabase(std::string_view path,
 	case DB_BACKEND_HONEY:
 	    throw InvalidArgumentError("Honey backend doesn't support "
 				       "updating existing databases");
-	case DB_BACKEND_CHERT:
-	    throw FeatureUnavailableError("Chert backend no longer supported");
 	case DB_BACKEND_INMEMORY:
 #ifdef XAPIAN_HAS_INMEMORY_BACKEND
 	    internal = new InMemoryDatabase();
