@@ -36,8 +36,6 @@
 #include "alignment_cast.h"
 #include "omassert.h"
 
-#ifndef WORDS_BIGENDIAN
-
 #if HAVE_DECL__BYTESWAP_USHORT || HAVE_DECL__BYTESWAP_ULONG
 # include <stdlib.h>
 #endif
@@ -82,8 +80,6 @@ inline uint64_t do_bswap(uint64_t value) {
 # endif
 }
 
-#endif
-
 template<typename UINT>
 inline UINT
 do_aligned_read(const unsigned char * ptr)
@@ -99,10 +95,10 @@ template<typename T, typename UINT>
 inline void
 do_aligned_write(unsigned char * ptr, T value)
 {
-    if (std::is_signed<T>::value) {
+    if (std::is_signed_v<T>) {
 	AssertRel(value, >=, 0);
     }
-    if (sizeof(T) > sizeof(UINT)) {
+    if constexpr(sizeof(T) > sizeof(UINT)) {
 	AssertEq(value, T(UINT(value)));
     }
     UINT v = UINT(value);
@@ -128,10 +124,10 @@ template<typename T, typename UINT>
 inline void
 do_unaligned_write(unsigned char * ptr, T value)
 {
-    if (std::is_signed<T>::value) {
+    if (std::is_signed_v<T>) {
 	AssertRel(value, >=, 0);
     }
-    if (sizeof(T) > sizeof(UINT)) {
+    if constexpr(sizeof(T) > sizeof(UINT)) {
 	AssertEq(value, T(UINT(value)));
     }
     UINT v = UINT(value);
