@@ -1,7 +1,7 @@
 /** @file
  * @brief Simple example program demonstrating query expansion.
  */
-/* Copyright (C) 2007,2010,2015 Olly Betts
+/* Copyright (C) 2007-2022 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * along with this program; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,14 +40,14 @@ try {
 	int rc = 1;
 	if (argv[1]) {
 	    if (strcmp(argv[1], "--version") == 0) {
-		cout << "simpleexpand" << endl;
+		cout << "simpleexpand\n";
 		exit(0);
 	    }
 	    if (strcmp(argv[1], "--help") == 0) {
 		rc = 0;
 	    }
 	}
-	cout << "Usage: " << argv[0] << " PATH_TO_DATABASE QUERY [-- [DOCID...]]" << endl;
+	cout << "Usage: " << argv[0] << " PATH_TO_DATABASE QUERY [-- [DOCID...]]\n";
 	exit(rc);
     }
 
@@ -82,14 +82,14 @@ try {
     qp.set_database(db);
     qp.set_stemming_strategy(Xapian::QueryParser::STEM_SOME);
     Xapian::Query query = qp.parse_query(query_string);
-    cout << "Parsed query is: " << query.get_description() << endl;
+    cout << "Parsed query is: " << query.get_description() << '\n';
 
     // Find the top 10 results for the query.
     enquire.set_query(query);
     Xapian::MSet matches = enquire.get_mset(0, 10, &rset);
 
     // Display the results.
-    cout << matches.get_matches_estimated() << " results found:" << endl;
+    cout << matches.get_matches_estimated() << " results found:\n";
 
     for (Xapian::MSetIterator i = matches.begin(); i != matches.end(); ++i) {
 	cout << i.get_rank() + 1 << ": " << i.get_weight() << " docid=" << *i
@@ -114,9 +114,9 @@ try {
     // List the terms.
     Xapian::ESetIterator t;
     for (t = eset.begin(); t != eset.end(); ++t) {
-	cout << *t << ": weight = " << t.get_weight() << endl;
+	cout << *t << ": weight = " << t.get_weight() << '\n';
     }
 } catch (const Xapian::Error &e) {
-    cout << e.get_description() << endl;
+    cout << e.get_description() << '\n';
     exit(1);
 }
