@@ -1,7 +1,8 @@
 /** @file
  *  @brief Class representing a list of search results
  */
-/* Copyright (C) 2015,2016,2019,2023 Olly Betts
+/* Copyright (C) 2015,2016,2017,2019,2023,2024 Olly Betts
+ * Copyright (C) 2018 Uppinder Chugh
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -14,9 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * along with this program; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #ifndef XAPIAN_INCLUDED_MSET_H
@@ -28,9 +28,11 @@
 
 #include <iterator>
 #include <string>
+#include <string_view>
 
 #include <xapian/attributes.h>
 #include <xapian/document.h>
+#include <xapian/error.h>
 #include <xapian/intrusive_ptr.h>
 #include <xapian/stem.h>
 #include <xapian/types.h>
@@ -47,11 +49,35 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
     // Helper function for fetch() methods.
     void fetch_(Xapian::doccount first, Xapian::doccount last) const;
 
+    /** Update the weight corresponding to the document indexed at
+     *  position i with wt.
+     *
+     *  The MSet's max_possible and max_attained are also updated.
+     *
+     *  This method must be called to update the weight of every document in
+     *  the MSet for i = 0 to mset.size() - 1 in ascending order to avoid
+     *  miscalculation of max_attained and max_possible.
+     *
+     *  @param i	MSet index to update
+     *  @param wt	new weight to assign to the document at index @a i
+     *
+     *  @since Added in Xapian 2.0.0.
+     */
+    void set_item_weight(Xapian::doccount i, double wt);
+
+#if 0 // FIXME: Need work before release.
+    /// Helper for diversify() method.
+    void diversify_(Xapian::doccount k,
+		    Xapian::doccount r,
+		    double factor1,
+		    double factor2);
+#endif
+
   public:
     /// Class representing the MSet internals.
     class Internal;
     /// @private @internal Reference counted internals.
-    Xapian::Internal::intrusive_ptr<Internal> internal;
+    Xapian::Internal::intrusive_ptr_nonnull<Internal> internal;
 
     /** Copying is allowed.
      *
@@ -65,13 +91,11 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
      */
     MSet & operator=(const MSet & o);
 
-#ifdef XAPIAN_MOVE_SEMANTICS
     /// Move constructor.
     MSet(MSet && o);
 
     /// Move assignment operator.
     MSet & operator=(MSet && o);
-#endif
 
     /** Default constructor.
      *
@@ -79,9 +103,98 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
      */
     MSet();
 
+    /** @private @internal Wrap an existing Internal. */
+    XAPIAN_VISIBILITY_INTERNAL
+    explicit MSet(Internal* internal_);
+
     /// Destructor.
     ~MSet();
 
+    /** Assigns new weights and updates MSet.
+     *
+     *  Dereferencing the Iterator should return a double.
+     *
+     *  The weights returned by the iterator are assigned to elements of
+     *  the MSet in rank order.
+     *
+     *  @param begin	Begin iterator.
+     *  @param end	End iterator.
+     *
+     *  @exception Xapian::InvalidArgument is thrown if the total number of
+     *		   elements in the input doesn't match the total number of
+     *		   documents in MSet.
+     *
+     *  @since Added in Xapian 2.0.0.
+     */
+    template<typename Iterator>
+    void replace_weights(Iterator first, Iterator last)
+    {
+	auto distance = last - first;
+	// Take care to compare signed and unsigned types both safely and
+	// without triggering compiler warnings.
+	if (distance < 0 ||
+	    (sizeof(distance) <= sizeof(Xapian::doccount) ?
+		Xapian::doccount(distance) != size() :
+		distance != static_cast<decltype(distance)>(size()))) {
+	    throw Xapian::InvalidArgumentError("Number of weights assigned "
+					       "doesn't match the number of "
+					       "items");
+	}
+	Xapian::doccount i = 0;
+	while (first != last) {
+	    set_item_weight(i, *first);
+	    ++i;
+	    ++first;
+	}
+    }
+
+    /** Sorts the list of documents in MSet according to their weights.
+     *
+     *  Use after calling MSet::replace_weights.
+     *
+     *  This invalidates any MSetIterator objects active on this MSet.
+     *
+     *  @since Added in Xapian 2.0.0.
+     */
+    void sort_by_relevance();
+
+#if 0 // FIXME: Need work before release.
+    /** Reorder MSet entries to diversify results.
+     *
+     *  The algorithm used is C²GLS-MPT as described in the paper: Scalable and
+     *  Efficient Web Search Result Diversification, Naini et al. 2016
+     *
+     *  @param  k	The number of MSet entries to make more diverse
+     *  @param  r	Number of documents from each cluster used for
+     *  		building topC
+     *  @param  lambda	Trade-off between relevance of top-k diversified
+     *  		document set and its similarity to the rest of the
+     *  		documents in the document match set.  Must be in
+     *  		the range [0,1] with 0 meaning no weighting to
+     *  		relevance of the diversified document set and 1
+     *  		allowing for full weighting to relevance of the
+     *  		diversified document set.
+     *  @param  b	Parameter for MPT, normally in the range [1,10]
+     *  @param  sigma_sqr	Parameter for MPT, normally in the range
+     *	  			[1e-6,1]
+     */
+    void diversify(Xapian::doccount k,
+		   Xapian::doccount r,
+		   double lambda = 0.5,
+		   double b = 5.0,
+		   double sigma_sqr = 1e-3) {
+	// Inline the argument value checks and the calculation of the scale
+	// factor for score_2 so the compiler can optimise in the case where
+	// some or all parameter values are compile-time constants.
+	if (r == 0)
+	    throw InvalidArgumentError("r must be > 0");
+	if (lambda < 0.0 || lambda > 1.0)
+	    throw InvalidArgumentError("lambda must be between 0 and 1");
+	if (k > 1)
+	    diversify_(k, r, lambda, (1.0 - lambda) * b * sigma_sqr * 2.0);
+    }
+#endif
+
     /** Convert a weight to a percentage.
      *
      *  The matching document with the highest weight will get 100% if it
@@ -115,15 +228,28 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
      *		gives the same answer as <code>db.get_termfreq(term)</code>
      *		(but is more efficient for query terms as it returns a
      *		value cached during the search.)
+     *
+     *  Since 2.0.0, this method returns 0 if called on an MSet which is
+     *  not associated with a database (which is consistent with
+     *  Database::get_termfreq() returning 0 when called on a Database
+     *  with no sub-databases); in earlier versions,
+     *  Xapian::InvalidOperationError was thrown in this case.
      */
-    Xapian::doccount get_termfreq(const std::string & term) const;
+    Xapian::doccount get_termfreq(std::string_view term) const;
 
     /** Get the term weight of a term.
      *
      *  @return	The maximum weight that @a term could have contributed to a
      *		document.
+     *
+     *  Since 2.0.0, this method returns 0.0 if called on an MSet which is
+     *  not associated with a database, or with a term which wasn't present
+     *  in the query (since in both cases the term contributes no weight to any
+     *  matching documents); in earlier versions, Xapian::InvalidOperationError
+     *  was thrown for the first case, and Xapian::InvalidArgumentError for the
+     *  second.
      */
-    double get_termweight(const std::string & term) const;
+    double get_termweight(std::string_view term) const;
 
     /** Rank of first item in this MSet.
      *
@@ -187,10 +313,10 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 
 	/** Generate n-grams for scripts without explicit word breaks.
 	 *
-	 *  Text in other scripts is split into words as normal.
-	 *
+         *  Text in other scripts is split into words as normal.
+         *
 	 *  Enable this option to highlight search results for queries parsed
-	 *  with the QueryParser::FLAG_NGRAMS flag.
+         *  with the QueryParser::FLAG_NGRAMS flag.
 	 *
 	 *  The TermGenerator::FLAG_NGRAMS flag needs to have been used at
 	 *  index time.
@@ -199,13 +325,13 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	 *  XAPIAN_CJK_NGRAM to a non-empty value (but doing so was deprecated
 	 *  in 1.4.11).
 	 *
-	 *  In 1.4.x this feature was specific to CJK (Chinese, Japanese and
-	 *  Korean), but in 2.0.0 it's been extended to other languages.  To
-	 *  reflect this change the new and preferred name is SNIPPET_NGRAMS,
-	 *  which was added as an alias for forward compatibility in Xapian
-	 *  1.4.23.  Use SNIPPET_CJK_NGRAM instead if you aim to support Xapian
-	 *  &lt; 1.4.23.
-	 *
+         *  In 1.4.x this feature was specific to CJK (Chinese, Japanese and
+         *  Korean), but in 2.0.0 it's been extended to other languages.  To
+         *  reflect this change the new and preferred name is SNIPPET_NGRAMS,
+         *  which was added as an alias for forward compatibility in Xapian
+         *  1.4.23.  Use SNIPPET_CJK_NGRAM instead if you aim to support Xapian
+         *  &lt; 1.4.23.
+         *
 	 *  @since Added in Xapian 1.4.23.
 	 */
 	SNIPPET_NGRAMS = 2048,
@@ -214,10 +340,25 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
 	 *
 	 *  Old name - use SNIPPET_NGRAMS instead unless you aim to support
 	 *  Xapian &lt; 1.4.23.
-	 *
+         *
 	 *  @since Added in Xapian 1.4.11.
 	 */
-	SNIPPET_CJK_NGRAM = SNIPPET_NGRAMS
+	SNIPPET_CJK_NGRAM = SNIPPET_NGRAMS,
+
+	/** Find word breaks for text in scripts without explicit word breaks.
+	 *
+	 *  Enable this option to highlight search results for queries parsed
+         *  with the QueryParser::FLAG_WORD_BREAKS flag.  Spans of text
+         *  written in such scripts are split into words using ICU (which uses
+         *  heuristics and/or dictionaries to do so).  Text in other scripts is
+         *  split into words as normal.
+	 *
+	 *  The TermGenerator::FLAG_WORD_BREAKS flag needs to have been used at
+	 *  index time.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	SNIPPET_WORD_BREAKS = 4096
     };
 
     /** Generate a snippet.
@@ -247,15 +388,15 @@ class XAPIAN_VISIBILITY_DEFAULT MSet {
      *
      *  And @a flags contains flags controlling behaviour.
      *
-     *  Added in 1.3.5.
+     *  @since Added in 1.3.5.
      */
-    std::string snippet(const std::string & text,
+    std::string snippet(std::string_view text,
 			size_t length = 500,
 			const Xapian::Stem & stemmer = Xapian::Stem(),
 			unsigned flags = SNIPPET_BACKGROUND_MODEL|SNIPPET_EXHAUSTIVE,
-			const std::string & hi_start = "<b>",
-			const std::string & hi_end = "</b>",
-			const std::string & omit = "...") const;
+			std::string_view hi_start = "<b>",
+			std::string_view hi_end = "</b>",
+			std::string_view omit = "...") const;
 
     /** Prefetch hint a range of items.
      *
@@ -536,62 +677,44 @@ class XAPIAN_VISIBILITY_DEFAULT MSetIterator {
     std::string get_description() const;
 };
 
-bool
-XAPIAN_NOTHROW(operator==(const MSetIterator &a, const MSetIterator &b));
-
 /// Equality test for MSetIterator objects.
 inline bool
-operator==(const MSetIterator &a, const MSetIterator &b) XAPIAN_NOEXCEPT
+operator==(const MSetIterator& a, const MSetIterator& b) noexcept
 {
     return a.off_from_end == b.off_from_end;
 }
 
-inline bool
-XAPIAN_NOTHROW(operator!=(const MSetIterator &a, const MSetIterator &b));
-
 /// Inequality test for MSetIterator objects.
 inline bool
-operator!=(const MSetIterator &a, const MSetIterator &b) XAPIAN_NOEXCEPT
+operator!=(const MSetIterator& a, const MSetIterator& b) noexcept
 {
     return !(a == b);
 }
 
-bool
-XAPIAN_NOTHROW(operator<(const MSetIterator &a, const MSetIterator &b));
-
 /// Inequality test for MSetIterator objects.
 inline bool
-operator<(const MSetIterator &a, const MSetIterator &b) XAPIAN_NOEXCEPT
+operator<(const MSetIterator& a, const MSetIterator& b) noexcept
 {
     return a.off_from_end > b.off_from_end;
 }
 
-inline bool
-XAPIAN_NOTHROW(operator>(const MSetIterator &a, const MSetIterator &b));
-
 /// Inequality test for MSetIterator objects.
 inline bool
-operator>(const MSetIterator &a, const MSetIterator &b) XAPIAN_NOEXCEPT
+operator>(const MSetIterator& a, const MSetIterator& b) noexcept
 {
     return b < a;
 }
 
-inline bool
-XAPIAN_NOTHROW(operator>=(const MSetIterator &a, const MSetIterator &b));
-
 /// Inequality test for MSetIterator objects.
 inline bool
-operator>=(const MSetIterator &a, const MSetIterator &b) XAPIAN_NOEXCEPT
+operator>=(const MSetIterator& a, const MSetIterator& b) noexcept
 {
     return !(a < b);
 }
 
-inline bool
-XAPIAN_NOTHROW(operator<=(const MSetIterator &a, const MSetIterator &b));
-
 /// Inequality test for MSetIterator objects.
 inline bool
-operator<=(const MSetIterator &a, const MSetIterator &b) XAPIAN_NOEXCEPT
+operator<=(const MSetIterator& a, const MSetIterator& b) noexcept
 {
     return !(b < a);
 }
