@@ -1,7 +1,7 @@
 /** @file
  * @brief Xapian::Query API class
  */
-/* Copyright (C) 2011-2025 Olly Betts
+/* Copyright (C) 2011-2026 Olly Betts
  * Copyright (C) 2008 Richard Boulton
  *
  * This program is free software; you can redistribute it and/or
@@ -15,8 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * along with this program; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #ifndef XAPIAN_INCLUDED_QUERY_H
@@ -27,6 +27,7 @@
 #endif
 
 #include <string>
+#include <string_view>
 
 #include <xapian/attributes.h>
 #include <xapian/intrusive_ptr.h>
@@ -36,22 +37,10 @@
 #include <xapian/types.h>
 #include <xapian/visibility.h>
 
-class QueryOptimiser; // FIXME
-
 namespace Xapian {
 
 class PostingSource;
 
-namespace Internal {
-    // You need a C++11 compiler to build Xapian 1.4.x, but we aim to support
-    // the headers being included without having to select C++11 mode.  That's
-    // become much less of an issue as most compilers have switched to at least
-    // C++11 by default, but it seems better not to change this mid release
-    // series, so we define our own version of std::enable_if<>.
-    template<bool, typename T = void> struct EnableIf { };
-    template<typename T> struct EnableIf<true, T> { typedef T type; };
-}
-
 /// Class representing a query.
 class XAPIAN_VISIBILITY_DEFAULT Query {
   public:
@@ -76,11 +65,12 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 
     /** A query matching all documents.
      *
-     *  This is a static instance of <code>Xapian::Query(std::string())</code>.
+     *  This is a static instance of
+     *  <code>Xapian::Query(std::string_view())</code>.
      *  If you are constructing Query objects which use @a MatchAll in
      *  different threads then the reference counting of the static object can
      *  get messed up by concurrent access so you should instead use
-     *  <code>Xapian::Query(std::string())</code> directly.
+     *  <code>Xapian::Query(std::string_view())</code> directly.
      */
     static const Xapian::Query MatchAll;
 
@@ -254,16 +244,30 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 *  is the maximum weight from any matching subquery (for OP_OR, it's
 	 *  the sum of the weights from the matching subqueries).
 	 *
-	 *  Added in Xapian 1.3.2.
+	 *  @since Added in Xapian 1.3.2.
 	 */
 	OP_MAX = 14,
 
 	/** Wildcard expansion.
 	 *
-	 *  Added in Xapian 1.3.3.
+	 *  @since Added in Xapian 1.3.3.
 	 */
 	OP_WILDCARD = 15,
 
+	/** Edit distance expansion.
+	 *
+	 *  Expand to terms within a specified edit distance of a target.
+	 *
+	 *  This works in a fairly similar way to OP_WILDCARD - the difference
+	 *  is that instead of expanding to terms matching a wildcard pattern,
+	 *  it expands to terms within a specified number of edits (insertion,
+	 *  deletion or substitution of a character, or transposition of two
+	 *  adjacent characters) of a specified target.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	OP_EDIT_DISTANCE = 16,
+
 	/** Construct an invalid query.
 	 *
 	 *  This can be useful as a placeholder - for example @a RangeProcessor
@@ -280,8 +284,8 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 
 	/** Value returned by get_type() for MatchAll or equivalent.
 	 *
-	 *  This is returned for any <code>Xapian::Query(std::string())</code>
-	 *  object.
+	 *  This is returned for any
+	 *  <code>Xapian::Query(std::string_view())</code> object.
 	 */
 	LEAF_MATCH_ALL,
 
@@ -298,13 +302,13 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 *  Xapian::WildcardError will be thrown when the query is actually
 	 *  run.
 	 */
-	WILDCARD_LIMIT_ERROR,
+	WILDCARD_LIMIT_ERROR = 0x00,
 	/** Stop expanding when OP_WILDCARD reaches its expansion limit.
 	 *
 	 *  This makes the wildcard expand to only the first N terms (sorted
 	 *  by byte order).
 	 */
-	WILDCARD_LIMIT_FIRST,
+	WILDCARD_LIMIT_FIRST = 0x01,
 	/** Limit OP_WILDCARD expansion to the most frequent terms.
 	 *
 	 *  If OP_WILDCARD would expand to more than its expansion limit, the
@@ -314,7 +318,28 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	 *  to evaluate than the full expansion, using only the most frequent
 	 *  terms tends to give better results too.
 	 */
-	WILDCARD_LIMIT_MOST_FREQUENT
+	WILDCARD_LIMIT_MOST_FREQUENT = 0x02,
+
+	/** @private @internal */
+	WILDCARD_LIMIT_MASK_ = 0x03,
+
+	/** Support * which matches 0 or more characters.
+	 *
+	 *  @since Added in Xapian 2.0.0
+	 */
+	WILDCARD_PATTERN_MULTI = 0x10,
+
+	/** Support ? which matches a single character.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	WILDCARD_PATTERN_SINGLE = 0x20,
+
+	/** Enable all supported glob-like features.
+	 *
+	 *  @since Added in Xapian 2.0.0.
+	 */
+	WILDCARD_PATTERN_GLOB = WILDCARD_PATTERN_MULTI|WILDCARD_PATTERN_SINGLE
     };
 
     /** Construct a query matching no documents.
@@ -326,7 +351,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *  so <code>Query() & q</code> is @c Query(), while
      *  <code>Query() | q</code> is @c q.
      */
-    XAPIAN_NOTHROW(Query()) { }
+    Query() noexcept { }
 
     /// Destructor.
     ~Query() { }
@@ -343,13 +368,11 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      */
     Query & operator=(const Query & o) { internal = o.internal; return *this; }
 
-#ifdef XAPIAN_MOVE_SEMANTICS
     /// Move constructor.
     Query(Query &&) = default;
 
     /// Move assignment operator.
     Query & operator=(Query &&) = default;
-#endif
 
     /** Construct a Query object for a term.
      *
@@ -360,7 +383,37 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *		    determine the order of terms obtained via
      *		    get_terms_begin(). (default: 0)
      */
-    Query(const std::string & term,
+    Query(const std::string& term,
+	  Xapian::termcount wqf = 1,
+	  Xapian::termpos pos = 0)
+	: Query(std::string_view(term), wqf, pos)
+    { }
+
+    /** Construct a Query object for a term.
+     *
+     *  @param term The term.  An empty string constructs a query matching
+     *		    all documents (@a MatchAll is a static instance of this).
+     *  @param wqf  The within-query frequency. (default: 1)
+     *  @param pos  The query position.  Currently this is mainly used to
+     *		    determine the order of terms obtained via
+     *		    get_terms_begin(). (default: 0)
+     */
+    Query(const char* term,
+	  Xapian::termcount wqf = 1,
+	  Xapian::termpos pos = 0)
+	: Query(std::string_view(term), wqf, pos)
+    { }
+
+    /** Construct a Query object for a term.
+     *
+     *  @param term The term.  An empty string constructs a query matching
+     *		    all documents (@a MatchAll is a static instance of this).
+     *  @param wqf  The within-query frequency. (default: 1)
+     *  @param pos  The query position.  Currently this is mainly used to
+     *		    determine the order of terms obtained via
+     *		    get_terms_begin(). (default: 0)
+     */
+    Query(std::string_view term,
 	  Xapian::termcount wqf = 1,
 	  Xapian::termpos pos = 0);
 
@@ -406,11 +459,16 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *  @param a	First term.
      *  @param b	Second term.
      */
-    Query(op op_, const std::string & a, const std::string & b)
+    template<typename S1, typename S2,
+	typename
+	std::enable_if<std::is_constructible<std::string_view, S1>::value &&
+		       std::is_constructible<std::string_view, S2>::value,
+		       bool>::type = true>
+    Query(op op_, S1 a, S2 b)
     {
 	init(op_, 2);
-	add_subquery(false, a);
-	add_subquery(false, b);
+	add_subquery(false, std::string_view(a));
+	add_subquery(false, std::string_view(b));
 	done();
     }
 
@@ -420,7 +478,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *  @param slot		The value slot to work over.
      *  @param range_limit	The limit of the range.
      */
-    Query(op op_, Xapian::valueno slot, const std::string & range_limit);
+    Query(op op_, Xapian::valueno slot, std::string_view range_limit);
 
     /** Construct a Query object for a value range.
      *
@@ -430,39 +488,99 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *  @param range_upper	Upper end of the range.
      */
     Query(op op_, Xapian::valueno slot,
-	  const std::string & range_lower, const std::string & range_upper);
+	  std::string_view range_lower, std::string_view range_upper);
 
-    /** Query constructor for OP_WILDCARD queries.
+    /** Query constructor for OP_EDIT_DISTANCE and OP_WILDCARD queries.
      *
-     *  @param op_	Must be OP_WILDCARD
-     *  @param pattern	The wildcard pattern - currently this is just a string
-     *			and the wildcard expands to terms which start with
-     *			exactly this string.
+     *  @param op_	Must be OP_EDIT_DISTANCE or OP_WILDCARD
+     *  @param pattern	The wildcard pattern (for OP_WILDCARD) or target string
+     *			(for OP_EDIT_DISTANCE).  See @a flags which affects
+     *		        how this pattern is interpreted.
      *	@param max_expansion	The maximum number of terms to expand to
      *				(default: 0, which means no limit)
-     *	@param max_type	How to enforce max_expansion - one of
-     *			@a WILDCARD_LIMIT_ERROR (the default),
-     *			@a WILDCARD_LIMIT_FIRST or
-     *			@a WILDCARD_LIMIT_MOST_FREQUENT.
-     *			When searching multiple databases, the expansion limit
-     *			is currently applied independently for each database,
-     *			so the total number of terms may be higher than the
-     *			limit.  This is arguably a bug, and may change in
-     *			future versions.
+     *	@param flags	Flags controlling aspects of the wildcarding - this
+     *			consists of a bitwise OR of:
+     *
+     *			* At most one of @a WILDCARD_LIMIT_ERROR (the default),
+     *			  @a WILDCARD_LIMIT_FIRST or
+     *			  @a WILDCARD_LIMIT_MOST_FREQUENT specifying how to
+     *			  enforce max_expansion.
+     *
+     *			  When searching multiple databases, the expansion
+     *			  limit is currently applied independently for each
+     *			  database, so the total number of terms may be higher
+     *			  than the limit.  This is arguably a bug, and may
+     *			  change in future versions.
+     *
+     *			* For OP_WILDCARD: Zero or more of
+     *			  @a WILDCARD_PATTERN_MULTI and
+     *			  @a WILDCARD_PATTERN_SINGLE, which specify whether
+     *			  '*' (matching zero or more characters) and '?'
+     *			  (matching exactly one character) are supported.
+     *			  If neither is specified, then a Xapian-1.4-compatible
+     *			  mode is used where the pattern matches terms which
+     *			  start with the pattern interpreted as a literal
+     *			  string.
+     *
      *	@param combiner The @a Query::op to combine the terms with - one of
      *			@a OP_SYNONYM (the default), @a OP_OR or @a OP_MAX.
+     *
+     *	For OP_WILDCARD: A leading wildcard won't match terms starting with an
+     *	ASCII capital letter, as this is assumed to be part of a term prefix.
      */
     Query(op op_,
-	  const std::string & pattern,
+	  std::string_view pattern,
 	  Xapian::termcount max_expansion = 0,
-	  int max_type = WILDCARD_LIMIT_ERROR,
+	  int flags = WILDCARD_LIMIT_ERROR,
 	  op combiner = OP_SYNONYM);
 
+    /** Query constructor for OP_EDIT_DISTANCE queries.
+     *
+     *  This form supports some additional parameters.
+     *
+     *  @param op_	Must be OP_EDIT_DISTANCE.
+     *  @param pattern	The target string.
+     *	@param max_expansion	The maximum number of terms to expand to
+     *				(default: 0, which means no limit)
+     *	@param flags	Flags controlling aspects of the wildcarding:
+     *
+     *			* At most one of @a WILDCARD_LIMIT_ERROR (the default),
+     *			  @a WILDCARD_LIMIT_FIRST or
+     *			  @a WILDCARD_LIMIT_MOST_FREQUENT specifying how to
+     *			  enforce max_expansion.
+     *
+     *			  When searching multiple databases, the expansion
+     *			  limit is currently applied independently for each
+     *			  database, so the total number of terms may be higher
+     *			  than the limit.  This is arguably a bug, and may
+     *			  change in future versions.
+     *
+     *	@param combiner The @a Query::op to combine the terms with - one of
+     *			@a OP_SYNONYM (the default), @a OP_OR or @a OP_MAX.
+     *	@param edit_distance
+     *			The maximum number of edits allowed between a term
+     *			and @a target (an edit is insertion, deletion or
+     *			substitution of a character, or transposition of two
+     *			adjacent characters).  Default: 2
+     *	@param min_prefix_len
+     *			The length in bytes of any initial substring of target
+     *			that is required to match exactly.  Default: 0
+     *
+     *  @since Added in Xapian 2.0.0.
+     */
+    Query(op op_,
+	  std::string_view pattern,
+	  Xapian::termcount max_expansion,
+	  int flags,
+	  op combiner,
+	  unsigned edit_distance,
+	  size_t min_prefix_len = 0);
+
     /** Construct a Query object from a begin/end iterator pair.
      *
      *  Dereferencing the iterator should return a Xapian::Query, a non-NULL
-     *  Xapian::Query*, a std::string or a type which converts to one of
-     *  these (e.g. const char*).
+     *  Xapian::Query*, a std::string_view or a type which converts to one of
+     *  these (e.g. std::string or const char*).
      *
      *  If begin == end then there are no subqueries and the resulting Query
      *  won't match anything.
@@ -474,10 +592,16 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *			number of subqueries as the window size (default: 0).
      */
     template<typename I,
-	typename T = typename std::iterator_traits<I>,
-	typename V = typename T::value_type,
-	typename Xapian::Internal::EnableIf<sizeof(V) != 1, bool>::type = true,
-	typename iterator_category = typename T::iterator_category>
+	typename std::enable_if<
+	    std::is_convertible<typename std::iterator_traits<I>::value_type,
+				Xapian::Query>::value ||
+	    std::is_convertible<typename std::iterator_traits<I>::value_type,
+				Xapian::Query*>::value ||
+	    std::is_convertible<typename std::iterator_traits<I>::value_type,
+				std::string_view>::value,
+	    bool>::type = true,
+	typename iterator_category =
+	    typename std::iterator_traits<I>::iterator_category>
     Query(op op_, I begin, I end, Xapian::termcount window = 0)
     {
 	if (begin != end) {
@@ -512,7 +636,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
     const TermIterator get_terms_begin() const;
 
     /// End iterator for terms in the query object.
-    const TermIterator XAPIAN_NOTHROW(get_terms_end() const) {
+    const TermIterator get_terms_end() const noexcept {
 	return TermIterator();
     }
 
@@ -526,16 +650,16 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
     const TermIterator get_unique_terms_begin() const;
 
     /// End iterator for unique terms in the query object.
-    const TermIterator XAPIAN_NOTHROW(get_unique_terms_end() const) {
+    const TermIterator get_unique_terms_end() const noexcept {
 	return TermIterator();
     }
 
     /** Return the length of this query object. */
-    Xapian::termcount XAPIAN_NOTHROW(get_length() const) XAPIAN_PURE_FUNCTION;
+    Xapian::termcount get_length() const noexcept XAPIAN_PURE_FUNCTION;
 
     /** Check if this query is Xapian::Query::MatchNothing. */
-    bool XAPIAN_NOTHROW(empty() const) {
-	return internal.get() == 0;
+    bool empty() const noexcept {
+	return !internal;
     }
 
     /** Serialise this object into a string. */
@@ -548,14 +672,26 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
      *				user-subclasses of Xapian::PostingSource
      *				(default: standard registry).
      */
-    static const Query unserialise(const std::string & serialised,
+    static const Query unserialise(std::string_view serialised,
 				   const Registry & reg = Registry());
 
     /** Get the type of the top level of the query. */
-    op XAPIAN_NOTHROW(get_type() const) XAPIAN_PURE_FUNCTION;
+    op get_type() const noexcept XAPIAN_PURE_FUNCTION;
 
     /** Get the number of subqueries of the top level query. */
-    size_t XAPIAN_NOTHROW(get_num_subqueries() const) XAPIAN_PURE_FUNCTION;
+    size_t get_num_subqueries() const noexcept XAPIAN_PURE_FUNCTION;
+
+    /** Get the wqf parameter of a leaf node.
+     *
+     *  @since Added in Xapian 2.0.0.
+     */
+    Xapian::termcount get_leaf_wqf() const;
+
+    /** Get the pos parameter of a leaf node.
+     *
+     *  @since Added in Xapian 2.0.0.
+     */
+    Xapian::termpos get_leaf_pos() const;
 
     /** Read a top level subquery.
       *
@@ -610,7 +746,7 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 	return (*this = Query(1.0 / factor, *this));
     }
 
-    /** @private @internal */
+    /// @private @internal Wrap an existing Internal.
     explicit Query(Internal * internal_) : internal(internal_) { }
 
     /** Construct with just an operator.
@@ -641,7 +777,15 @@ class XAPIAN_VISIBILITY_DEFAULT Query {
 
     void add_subquery(bool positional, const Xapian::Query & subquery);
 
-    void add_subquery(bool, const std::string & subquery) {
+    void add_subquery(bool, const std::string& subquery) {
+	add_subquery(false, Xapian::Query(subquery));
+    }
+
+    void add_subquery(bool, const char* subquery) {
+	add_subquery(false, Xapian::Query(subquery));
+    }
+
+    void add_subquery(bool, std::string_view subquery) {
 	add_subquery(false, Xapian::Query(subquery));
     }
 
@@ -720,7 +864,7 @@ class InvertedQuery_ {
     InvertedQuery_(const InvertedQuery_ & o) : query(o.query) { }
 
     operator Query() const {
-	return Query(Query::OP_AND_NOT, Query(std::string()), query);
+	return Query(Query::OP_AND_NOT, Query(std::string_view{}), query);
     }
 
     friend const InvertedQuery_ operator~(const Query &q);
@@ -763,42 +907,79 @@ namespace Internal {
 class AndContext;
 class OrContext;
 class XorContext;
+
+struct PostListAndEstimate;
+class QueryOptimiser;
+struct TermFreqs;
 }
 
 /** @private @internal */
 class Query::Internal : public Xapian::Internal::intrusive_base {
   public:
-    XAPIAN_NOTHROW(Internal()) { }
+    Internal() noexcept { }
 
     virtual ~Internal();
 
-    virtual PostingIterator::Internal * postlist(QueryOptimiser * qopt, double factor) const = 0;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual Xapian::Internal::PostListAndEstimate
+    postlist(Xapian::Internal::QueryOptimiser* qopt,
+	     double factor,
+	     Xapian::Internal::TermFreqs* termfreqs) const = 0;
 
-    virtual void postlist_sub_and_like(Xapian::Internal::AndContext& ctx,
-				       QueryOptimiser * qopt,
-				       double factor) const;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual bool
+    postlist_sub_and_like(Xapian::Internal::AndContext& ctx,
+			  Xapian::Internal::QueryOptimiser* qopt,
+			  double factor,
+			  Xapian::Internal::TermFreqs* termfreqs) const;
 
-    virtual void postlist_sub_or_like(Xapian::Internal::OrContext& ctx,
-				      QueryOptimiser * qopt,
-				      double factor) const;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual void
+    postlist_sub_bool_or_like(Xapian::Internal::OrContext& ctx,
+			      Xapian::Internal::QueryOptimiser* qopt,
+			      Xapian::Internal::TermFreqs* termfreqs) const;
 
-    virtual void postlist_sub_xor(Xapian::Internal::XorContext& ctx,
-				  QueryOptimiser * qopt,
-				  double factor) const;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual void
+    postlist_sub_or_like(Xapian::Internal::OrContext& ctx,
+			 Xapian::Internal::QueryOptimiser* qopt,
+			 double factor,
+			 Xapian::Internal::TermFreqs* termfreqs,
+			 bool keep_zero_weight = true) const;
 
-    virtual termcount XAPIAN_NOTHROW(get_length() const) XAPIAN_PURE_FUNCTION;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual void
+    postlist_sub_xor(Xapian::Internal::XorContext& ctx,
+		     Xapian::Internal::QueryOptimiser* qopt,
+		     double factor,
+		     Xapian::Internal::TermFreqs* termfreqs) const;
 
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual termcount get_length() const noexcept XAPIAN_PURE_FUNCTION;
+
+    XAPIAN_VISIBILITY_INTERNAL
     virtual void serialise(std::string & result) const = 0;
 
-    static Query::Internal * unserialise(const char ** p, const char * end, const Registry & reg);
+    XAPIAN_VISIBILITY_INTERNAL
+    static Query::Internal* unserialise(const char** p, const char* end,
+					const Registry& reg);
 
-    virtual Query::op XAPIAN_NOTHROW(get_type() const) XAPIAN_PURE_FUNCTION = 0;
-    virtual size_t XAPIAN_NOTHROW(get_num_subqueries() const) XAPIAN_PURE_FUNCTION;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual Query::op get_type() const noexcept XAPIAN_PURE_FUNCTION = 0;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual size_t get_num_subqueries() const noexcept XAPIAN_PURE_FUNCTION;
+    XAPIAN_VISIBILITY_INTERNAL
     virtual const Query get_subquery(size_t n) const;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual termcount get_wqf() const;
+    XAPIAN_VISIBILITY_INTERNAL
+    virtual termpos get_pos() const;
 
+    XAPIAN_VISIBILITY_INTERNAL
     virtual std::string get_description() const = 0;
 
     // Pass argument as void* to avoid need to include <vector>.
+    XAPIAN_VISIBILITY_INTERNAL
     virtual void gather_terms(void * void_terms) const;
 };
 
@@ -809,7 +990,7 @@ Query::operator&=(const Query & o)
 	// q &= empty_query sets q to empty_query.
 	*this = o;
     } else if (this != &o &&
-	       internal.get() &&
+	       internal &&
 	       internal->_refs == 1 &&
 	       get_type() == OP_AND) {
 	// Appending a subquery to an existing AND.
@@ -826,7 +1007,7 @@ Query::operator|=(const Query & o)
     if (o.empty()) {
 	// q |= empty_query is a no-op.
     } else if (this != &o &&
-	       internal.get() &&
+	       internal &&
 	       internal->_refs == 1 &&
 	       get_type() == OP_OR) {
 	// Appending a subquery to an existing OR.
@@ -845,7 +1026,7 @@ Query::operator^=(const Query & o)
     } else if (internal.get() == o.internal.get()) {
 	// q ^= q gives MatchNothing.
 	internal = NULL;
-    } else if (internal.get() &&
+    } else if (internal &&
 	       internal->_refs == 1 &&
 	       get_type() == OP_XOR) {
 	// Appending a subquery to an existing XOR.
