@@ -21,8 +21,8 @@ const char * dummy[] = {
 "// GNU General Public License for more details.",
 "//",
 "// You should have received a copy of the GNU General Public License",
-"// along with this program; if not, write to the Free Software",
-"// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA",
+"// along with this program; if not, see",
+"// <https://www.gnu.org/licenses/>.",
 "",
 "#ifndef XAPIAN_INCLUDED_VERSION_H",
 "#define XAPIAN_INCLUDED_VERSION_H",
