@@ -14,9 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * along with this program; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -67,11 +66,12 @@ DEFINE_TESTCASE(qpsynonympartial1, synonyms) {
 	{ NULL, NULL }
     };
 
-    Xapian::WritableDatabase db(get_writable_database());
-    db.add_synonym("hello", "hi");
-    db.add_synonym("hello", "howdy");
-    db.commit();
-
+    Xapian::Database db = get_database("qpsynonympartial1",
+				       [](Xapian::WritableDatabase& wdb,
+					  const string&) {
+					   wdb.add_synonym("hello", "hi");
+					   wdb.add_synonym("hello", "howdy");
+				       });
     Xapian::Enquire enquire(db);
     Xapian::QueryParser qp;
     Xapian::Stem stemmmer("english");
