@@ -14,8 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * along with this program; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -90,7 +90,7 @@ FDTracker::init()
 	if (!entry) {
 	    if (errno == 0)
 		break;
-	    cout << "readdir failed: " << errno_to_string(errno) << endl;
+	    cout << "readdir failed: " << errno_to_string(errno) << '\n';
 	    exit(1);
 	}
 
@@ -120,7 +120,7 @@ FDTracker::check()
 	if (!entry) {
 	    if (errno == 0)
 		break;
-	    cout << "readdir failed: " << errno_to_string(errno) << endl;
+	    cout << "readdir failed: " << errno_to_string(errno) << '\n';
 	    exit(1);
 	}
 
