@@ -2,7 +2,7 @@
  * @brief Long-running "soak" tests for Xapian.
  */
 /* Copyright (C) 2010 Richard Boulton
- * Copyright (C) 2011,2013 Olly Betts
+ * Copyright (C) 2011,2013,2022 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -15,9 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
- * USA
+ * along with this program; if not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -28,36 +27,25 @@
 #include "testrunner.h"
 #include "testsuite.h"
 
-// random() and srandom() aren't in <cstdlib> with Sun's compiler.
-#include <stdlib.h>
+#include <random>
 
 using namespace std;
 
-unsigned int g_random_seed;
+static unsigned int g_random_seed;
+
+static mt19937 gen;
 
 extern unsigned int initrand()
 {
     tout << "Setting random seed to " << g_random_seed << "\n";
-#if defined HAVE_SRANDOM && defined HAVE_RANDOM
-    srandom(g_random_seed);
-#else
-    srand(g_random_seed);
-#endif
+    gen.seed(g_random_seed);
     return g_random_seed;
 }
 
 extern unsigned int randint(unsigned int s)
 {
-#if defined HAVE_SRANDOM && defined HAVE_RANDOM
-    unsigned long int r = static_cast<unsigned long int>(random());
-    r = r % static_cast<unsigned long int>(s);
-    return static_cast<unsigned int>(r);
-#else
-    unsigned int r = static_cast<unsigned long int>(rand());
-    // The low order bits have lousy randomness on some platforms.
-    r = r / (RAND_MAX / s);
-    return r;
-#endif
+    uniform_int_distribution<> distribution(0, s - 1);
+    return static_cast<unsigned int>(distribution(gen));
 }
 
 class SoakTestRunner : public TestRunner
